@@ -28,6 +28,10 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 
+#ifdef HAVE_BASENAME
+#include <libgen.h>
+#endif
+
 #ifdef ENABLE_XEMBED
 #include <gdk/gdkx.h>
 #endif /* ENABLE_XEMBED */
@@ -35,6 +39,18 @@
 #define DEFAULT_WIDTH  240
 #define DEFAULT_HEIGHT 240
 
+static char * _rsvg_basename(const char * file)
+{
+#ifdef HAVE_BASENAME
+	if(file && *file) {
+		char * file_dup = g_strdup(file);
+		return basename(file_dup);
+	}
+#endif
+
+	return NULL;
+}
+
 typedef struct _ViewerCbInfo ViewerCbInfo;
 struct _ViewerCbInfo
 {
@@ -183,11 +199,15 @@ print_pixbuf (GObject * ignored, gpointer user_data)
 #if GTK_CHECK_VERSION(2,4,0)
 
 static char *
-save_file (const char * title, GtkWidget * parent)
+save_file (const char * title, 
+		   const char * suggested_filename,
+		   GtkWidget * parent,
+		   int *success)
 {
 	GtkWidget *dialog;
 	char *filename = NULL;
 
+	*success = 0;
 	dialog = gtk_file_chooser_dialog_new (title,
 										  GTK_WINDOW (parent),
 										  GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -195,10 +215,14 @@ save_file (const char * title, GtkWidget * parent)
 										  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 										  NULL);
 
+	if(suggested_filename && *suggested_filename) {
+		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER (dialog), suggested_filename);
+	}
+
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-		{
-			
+		{			
 			filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+			*success = 1;
 		}
 	
 	gtk_widget_destroy (dialog);
@@ -209,17 +233,28 @@ save_file (const char * title, GtkWidget * parent)
 #else
 
 static char *
-save_file (const char * title, GtkWidget * parent)
+save_file (const char * title, 
+		   const char * suggested_filename,
+		   GtkWidget * parent,
+		   int *success)
 {
 	GtkWidget * filesel;
 	char * filename = NULL;
-			
+	
+	*success = 0;
 	filesel = gtk_file_selection_new (title);
 	gtk_window_set_transient_for(GTK_WINDOW(filesel), parent);
+
+	if(suggested_filename && *suggested_filename) {
+		char * utf8_suggestion = g_filename_from_utf8(suggested_filename);
+		gtk_file_selection_set_filename(GTK_FILE_SELECTION (filesel), utf8_suggestion);
+		g_free(utf8_suggestion);
+	}
 	
 	if (gtk_dialog_run (GTK_DIALOG (filesel)) == GTK_RESPONSE_OK)
 		{			
 			filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel)));
+			*success = 1;
 		}
 
 	gtk_widget_destroy (dialog);
@@ -233,10 +268,19 @@ static void
 save_pixbuf (GObject * ignored, gpointer user_data)
 {
 	ViewerCbInfo * info = (ViewerCbInfo *)user_data;
-	char * filename;
+	char * filename, *base_name, *filename_suggestion;
+	int success = 0;
+
+	base_name = _rsvg_basename(info->base_uri);
+	if(base_name)
+		filename_suggestion = g_strdup_printf("%s.png", base_name);
+	else
+		filename_suggestion = NULL;
+
+	filename = save_file (_("Save SVG as PNG"), filename_suggestion, info->window, &success);
+	g_free(base_name);
+	g_free(filename_suggestion);
 
-	filename = save_file (_("Save SVG as PNG"), info->window);
-	
 	if (filename) 
 		{
 			GError * err = NULL;
@@ -262,7 +306,7 @@ save_pixbuf (GObject * ignored, gpointer user_data)
 
 			g_free (filename);
 		} 
-	else 
+	else if(success)
 		{
 			GtkWidget * errmsg;
 			
@@ -282,14 +326,18 @@ static void
 save_svg (GObject * ignored, gpointer user_data)
 {
 	ViewerCbInfo * info = (ViewerCbInfo *)user_data;
-	char * filename;
-	
-	filename = save_file (_("Save SVG"), info->window);
+	char * filename, *base_name;
+	int success = 0;
+
+	base_name = _rsvg_basename(info->base_uri);
+	filename = save_file (_("Save SVG"), base_name, info->window, &success);
+	g_free(base_name);
 
 	if (filename) 
 		{
 			FILE * fp;
 			
+			/* todo: make this support gnome vfs */
 			fp = fopen(filename, "wb");
 			if (!fp) 
 				{
@@ -340,7 +388,7 @@ save_svg (GObject * ignored, gpointer user_data)
 
 			g_free (filename);
 		} 
-	else 
+	else if(success)
 		{
 			GtkWidget * errmsg;
 					
@@ -546,7 +594,6 @@ view_pixbuf (ViewerCbInfo * info, int xid, const char * color)
 int 
 main (int argc, char **argv)
 {
-	FILE * in;
 	GError * err = NULL;
 	poptContext popt_context;
 	double x_zoom = 1.0;
@@ -651,14 +698,42 @@ main (int argc, char **argv)
 
 	size_data.keep_aspect_ratio = bKeepAspect;
 	
-	if(from_stdin)
-		in = stdin;
-	else {
-		in = fopen (args[0], "rb");
-		base_uri = (char *)args[0];
+	if(!from_stdin) {
+		if (base_uri == NULL)
+			base_uri = (char *)args[0];
+	
+		info.svg_bytes = _rsvg_acquire_xlink_href_resource(args[0], base_uri, NULL);
+	} else {
+		info.svg_bytes = g_byte_array_new ();
+
+		for (;;)
+			{
+				char buf[1024 * 8];
+				size_t nread = fread (buf, 1, sizeof(buf), stdin);
+			
+				if (nread > 0)
+					g_byte_array_append (info.svg_bytes, buf, nread);
+				
+				if (nread < sizeof (buf))
+				{
+					if (ferror (stdin))
+						{
+							g_critical (_("Error reading\n"));
+							g_byte_array_free (info.svg_bytes, TRUE);
+							poptFreeContext (popt_context);
+							fclose(stdin);
+							
+							return 1;
+						}
+					else if (feof(stdin))
+						break;
+				}
+			}
+		
+		fclose(stdin);		
 	}
 
-	if(!in)
+	if(!info.svg_bytes || !info.svg_bytes->len)
 		{
 			g_critical (_("Couldn't open %s\n"), args[0]);
 			poptFreeContext (popt_context);
@@ -667,33 +742,6 @@ main (int argc, char **argv)
 		}
 
 	info.base_uri = base_uri;
-	info.svg_bytes = g_byte_array_new ();
-
-	for (;;)
-		{
-			char buf[1024 * 8];
-			size_t nread = fread (buf, 1, sizeof(buf), in);
-			
-			if (nread > 0)
-				g_byte_array_append (info.svg_bytes, buf, nread);
-			
-			if (nread < sizeof (buf))
-				{
-					if (ferror (in))
-						{
-							g_critical (_("Error reading\n"));
-							g_byte_array_free (info.svg_bytes, TRUE);
-							poptFreeContext (popt_context);
-							fclose(in);
-							
-							return 1;
-						}
-					else if (feof(in))
-						break;
-				}
-		}
-	
-	fclose(in);
 
 	poptFreeContext (popt_context);
 
